/* Shopper
 * Copyright (C) 2008 David Greaves
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

//#define DEBUG_SHOPPER 1
#include "shopper.h"           // automake, i8n, gettext
#include "ListView.h"
#include "CategoryView.h"
#include "ItemDialog.h"
#include "LabelEntry.h"
#include <QtGui>
#include <iostream>
#include "GestureWatcher.h"

using namespace std;

namespace Shopper
{
// This class displays and manages a list.
//
// It creates ItemViews dynamically
// When the mode changes it updates the display
// When the active category changes, it updates the display
//
	struct ListView::Private {
		QHash<Category*, CategoryView*> catMap;
		Shopper::List        *mylist;
		
		QFrame               *contents; // This widget has a layout and contains all the items
		QVBoxLayout          *lvBox;     // and is scrolled by this widget
	};


// class ListView: public QScrollArea
	ListView::ListView(List &l, QWidget *parent) :
		QFingerScrollArea(parent)
	{
		p = new Private;
		p->mylist =&l;
		p->contents = NULL;
		p->lvBox = NULL;
			
//		setBackgroundRole(QPalette::Light);
		setWidgetResizable(true);
		QSizePolicy sizePolicy(QSizePolicy::Ignored, QSizePolicy::Preferred);
		sizePolicy.setHorizontalStretch(1);
		setSizePolicy(sizePolicy);

		setup();
		zoomOut();
		updateVisibility();

		// Handle changes to the data structure
		connect(&l, SIGNAL(category_added(Category*)),
				this, SLOT(add_cat(Category*)));
		connect(&l, SIGNAL(category_removed(Category*)),
				this, SLOT(del_cat(Category*)));


		
		// Now connect the List's changed signals
		// When the mode changes
		connect(&l, SIGNAL(state_changed()),
				this, SLOT(list_changed()));

		// When the active category changes
		connect(&l, SIGNAL(active_category_changed()),
				this, SLOT(list_changed()));

		// Issued on name change
		connect(&l, SIGNAL(changed()),
				this, SLOT(list_changed()));
	}

	ListView::~ListView()
	{
		delete p;
	}

	void ListView::filter_by(Category &c)  // Only view certain categories
	{
		p->mylist->make_category_active(c);
	}

	void ListView::list_changed()
	{
		_ENTER;
		DEBUG("Notified of a list_change");
		updateVisibility();
	}
	
    // FIX: This should specify position
	void ListView::add_cat(Shopper::Category *c)
	{
//		CategoryView *cw;
//		cw = new CategoryView(*c, *(p->mylist), p->contents);
//		p->lvBox->addWidget(cw, 1);
//		p->catMap[c] = cw;
		setup(); // FIX: change Shopper::List to be based on QList and do this properly
	}

	void ListView::del_cat(Shopper::Category *c)
	{
		QWidget *w = p->catMap[c];
		p->lvBox->removeWidget(w);
		p->catMap.remove(c);
		delete w;
		updateVisibility(); // FIX: change Shopper::List to be based on QList and do this properly
	}

	// Creates the structure of view widgets
	void ListView::setup()
	{
		if (p->contents!=NULL) p->contents->deleteLater(); // also deletes p->lvBox
		p->contents = new QFrame;
		p->lvBox = new QVBoxLayout;
		p->contents->setLayout(p->lvBox);
		p->contents->setLineWidth(3);

		// Scans all the categories in the Shopper::List and creates views
		for(Shopper::List::pCategoryIter cI = p->mylist->categoriesI();
			cI != p->mylist->categoriesEnd(); ++cI)
		{
			Shopper::Category *c = *cI;
//			add_cat(c);
			{ // FIX: This should be 'add_cat()'
				CategoryView *cw;
				cw = new CategoryView(*c, *(p->mylist), p->contents);
				p->lvBox->addWidget(cw, 1);
				p->catMap[c] = cw;
			}
		}

		p->lvBox->addStretch(1);
		QSizePolicy sizePolicy(QSizePolicy::Ignored, QSizePolicy::Preferred);
		sizePolicy.setHorizontalStretch(1);
		p->contents->setSizePolicy(sizePolicy);
		setWidget(p->contents);
	}
	
	// iterate through all widgets and hide/show them
	void ListView::updateVisibility()
	{
		_ENTER;
		CategoryView *cw;
		p->contents->hide();
		for(Shopper::List::pCategoryIter cI = p->mylist->categoriesI();
			cI != p->mylist->categoriesEnd(); ++cI)
		{
			Shopper::Category *c = *cI;
			cw = p->catMap[c];
			bool isActive = p->mylist->is_category_active(*c);
			if (isActive) {
				cw->updateVisibility();
				cw->show();
			} else {
				cw->hide();
			}
		}
		p->contents->show();
	}

	void ListView::zoomIn()
	{
		int max1 = verticalScrollBar()->maximum();
		int s = verticalScrollBar()->value();
		DEBUG("Scrolled to " << s << "/" <<max1);
		for(Shopper::List::pCategoryIter cI = p->mylist->categoriesI();
			cI != p->mylist->categoriesEnd(); ++cI)
		{
				p->catMap[*cI]->zoomIn();
		}
		qApp->sendPostedEvents();
		qApp->processEvents();
		qApp->processEvents();
		int max2 = verticalScrollBar()->maximum();
//		DEBUG("given " << max2 << " going to " << s*max2/max1);
		if (max1) verticalScrollBar()->setValue(s*max2/max1);
	}
	void ListView::zoomOut()
	{
		int max1 = verticalScrollBar()->maximum();
		int s = verticalScrollBar()->value();
		DEBUG("Scrolled to " << s << "/" <<max1);
		for(Shopper::List::pCategoryIter cI = p->mylist->categoriesI();
			cI != p->mylist->categoriesEnd(); ++cI)
		{
				p->catMap[*cI]->zoomOut();
		}
		qApp->sendPostedEvents();
		qApp->processEvents();
		qApp->processEvents();
		int max2 = verticalScrollBar()->maximum();
//		DEBUG("given " << max2 << " going to " << s*max2/max1);
		if (max1) verticalScrollBar()->setValue(s*max2/max1);
	}
}

